package demo.config;

import javax.sql.DataSource;

import org.seasar.doma.jdbc.DomaAbstractConfig;
import org.seasar.doma.jdbc.JdbcLogger;
import org.seasar.doma.jdbc.RequiresNewController;
import org.seasar.doma.jdbc.dialect.Dialect;
import org.seasar.doma.jdbc.dialect.HsqldbDialect;
import org.seasar.extension.datasource.DataSourceFactory;
import org.seasar.extension.datasource.impl.SelectableDataSourceProxy;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.SingletonS2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;

public class AppConfig extends DomaAbstractConfig {

    protected static final JdbcLogger jdbcLogger = new CommonsLogger();

    protected static final RequiresNewController requiresNewController = new S2RequiresNewController();

    private static final Dialect dialect = new HsqldbDialect();

    @Override
    public String dataSourceName() {
        S2Container singletonContainer = SingletonS2ContainerFactory
                .getContainer();
        if (getDataSource() instanceof SelectableDataSourceProxy
                && singletonContainer.hasComponentDef(DataSourceFactory.class)) {
            DataSourceFactory dataSourceFactory = (DataSourceFactory) singletonContainer
                    .getComponent(DataSourceFactory.class);
            return dataSourceFactory.getSelectableDataSourceName();
        }
        return super.dataSourceName();
    }

    @Override
    public DataSource dataSource() {
        return getDataSource();
    }

    protected DataSource getDataSource() {
        return SingletonS2Container.getComponent(DataSource.class);
    }

    @Override
    public Dialect dialect() {
        return dialect;
    }

    @Override
    public RequiresNewController requiresNewController() {
        return requiresNewController;
    }

}
